@@ -61,7 +61,6 @@ class BuildTarget:
         self.install_target = 'install'
         self.cmake_ndk_toolchain   = '' # Custom Android toolchain file for this target only
         self.cmake_raspi_toolchain = '' # Custom Raspberry toolchain file for this target only
-        self.cmake_oclea_toolchain = '' # Custom Oclea toolchain file for this target only
         self.cmake_ios_toolchain   = '' # Custom iOS toolchain file for this target only
         self.cmake_opts       = []
         self.cmake_cxxflags   = dict()
@@ -159,7 +158,7 @@ class BuildTarget:
         self.config.set_artifactory_ftp(ftp_url=ftp_url, auth=auth)
 
 
-    def add_local(self, name, source_dir, mamafile=None, always_build=False, args=[]):
+    def add_local(self, name, source_dir, mamafile=None, always_build=False, args=[]) -> BuildDependency:
         """
         Add a local dependency. This can be a git submodule or just some local folder.
         which contains its own CMakeLists.txt.
@@ -179,11 +178,12 @@ class BuildTarget:
         self.add_local('avdecoder', 'lib/avdecoder', always_build=True)
         ```
         """
-        if self.dep.from_artifactory: return # ignore if already loaded from artifactory
-        self.dep.add_child(LocalSource(name, source_dir, mamafile, always_build, args))
+        if self.dep.from_artifactory: # already loaded from artifactory?
+            return self.get_dependency(name)
+        return self.dep.add_child(LocalSource(name, source_dir, mamafile, always_build, args))
 
 
-    def add_git(self, name, git_url, git_branch='', git_tag='', mamafile=None, args=[]):
+    def add_git(self, name, git_url, git_branch='', git_tag='', mamafile=None, args=[]) -> BuildDependency:
         """
         Add a remote GIT dependency.
         The dependency will be cloned and updated according to mamabuild.
@@ -202,11 +202,12 @@ class BuildTarget:
                      git_branch='3.4', mamafile='mama/opencv_cfg.py')
         ```
         """
-        if self.dep.from_artifactory: return # ignore if already loaded from artifactory
-        self.dep.add_child(Git(name, git_url, git_branch, git_tag, mamafile, args))
+        if self.dep.from_artifactory: # already loaded from artifactory?
+            return self.get_dependency(name)
+        return self.dep.add_child(Git(name, git_url, git_branch, git_tag, mamafile, args))
 
 
-    def add_artifactory_pkg(self, name, version='latest', fullname=None):
+    def add_artifactory_pkg(self, name, version='latest', fullname=None) -> BuildDependency:
         """
         Adds an Artifactory only dependency.
         The dependency will be downloaded from the artifactory url.
@@ -227,11 +228,12 @@ class BuildTarget:
         self.add_artifactory_pkg('mylib', fullname='mylib-linux-x64-release-df76b66')
         ```
         """
-        if self.dep.from_artifactory: return # ignore if already loaded from artifactory
-        self.dep.add_child(ArtifactoryPkg(name, version=version, fullname=fullname))
+        if self.dep.from_artifactory: # already loaded from artifactory?
+            return self.get_dependency(name)
+        return self.dep.add_child(ArtifactoryPkg(name, version=version, fullname=fullname))
 
 
-    def get_dependency(self, name):
+    def get_dependency(self, name: str) -> BuildDependency:
         """
         Finds a child dependency by name.
         ```
@@ -1190,13 +1192,17 @@ class BuildTarget:
 
 
     def try_automatic_artifactory_fetch(self):
-        is_deploy = self.config.deploy or self.config.upload
-        is_target = not self.config.rebuild and self.is_current_target()
+        if not self.dep.can_fetch_artifactory(print=True, which='AUTO'):
+            return None
+
         # auto-fetch if:
-        # - not a deploy task
-        # - not the current build target eg `mama build this_target`
         # - is not the root project, roots should never be fetched
-        if not is_deploy and not is_target and not self.dep.is_root:
+        # - not a deploy/upload task
+        # - not the current build target eg `mama build this_target`
+        is_target = not self.dep.is_root and self.is_current_target()
+        is_deploy = self.config.deploy or self.config.upload
+        is_build = self.config.build
+        if is_target and not is_deploy and not is_build:
             fetched, _ = artifactory_fetch_and_reconfigure(self) # this will reconfigure packaging
             return fetched
         return None
@@ -1205,7 +1211,7 @@ class BuildTarget:
     ## TODO: move all of this into a new utility
     def _execute_build_tasks(self):
         can_build = not self.dep.nothing_to_build
-        if can_build and self.dep.should_rebuild:
+        if can_build and self.dep.should_rebuild and not self.dep.from_artifactory:
             self.configure() # user customization
             if can_build:
                 fetched = self.try_automatic_artifactory_fetch()
