@@ -240,8 +240,9 @@ class Git(DepSource):
                 if dep.config.verbose:
                     console(output, end='')
             else:
-                console(f'\r  - Target {dep.name: <16} CLONE FAILED                   ', color=Color.RED)
+                console(f'\r  - Target {dep.name: <16} CLONE FAILED ({result})              ', color=Color.RED)
                 console(output, end='')
+                raise RuntimeError(f'Target {self.name} clone failed: {cmd}')
 
 
     def clone_or_pull(self, dep: BuildDependency, wiped=False):
